@@ -27,27 +27,27 @@
  * intentionally made simple.
  */
 #ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif /* HAVE_CONFIG_H */
+#  include <config.h>
+#endif /* defined(HAVE_CONFIG_H) */
 
 #include <inttypes.h>
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif /* HAVE_UNISTD_H */
+#  include <unistd.h>
+#endif /* defined(HAVE_UNISTD_H) */
 #ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif /* HAVE_FCNTL_H */
+#  include <fcntl.h>
+#endif /* defined(HAVE_FCNTL_H) */
 #include <sys/types.h>
 #ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#endif /* HAVE_SYS_SOCKET_H */
+#  include <sys/socket.h>
+#endif /* defined(HAVE_SYS_SOCKET_H) */
 #ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif /* HAVE_NETDB_H */
+#  include <netdb.h>
+#endif /* defined(HAVE_NETDB_H) */
 #ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif /* HAVE_NETINET_IN_H */
+#  include <netinet/in.h>
+#endif /* defined(HAVE_NETINET_IN_H) */
 #include <netinet/tcp.h>
 #include <poll.h>
 #include <signal.h>
@@ -56,6 +56,7 @@
 #include <string.h>
 #include <errno.h>
 
+#define NGHTTP2_NO_SSIZE_T
 #include <nghttp2/nghttp2.h>
 
 #include <openssl/ssl.h>
@@ -66,14 +67,14 @@ enum { IO_NONE, WANT_READ, WANT_WRITE };
 
 #define MAKE_NV(NAME, VALUE)                                                   \
   {                                                                            \
-    (uint8_t *)NAME, (uint8_t *)VALUE, sizeof(NAME) - 1, sizeof(VALUE) - 1,    \
-        NGHTTP2_NV_FLAG_NONE                                                   \
+    (uint8_t *)NAME,   (uint8_t *)VALUE,     sizeof(NAME) - 1,                 \
+    sizeof(VALUE) - 1, NGHTTP2_NV_FLAG_NONE,                                   \
   }
 
 #define MAKE_NV_CS(NAME, VALUE)                                                \
   {                                                                            \
-    (uint8_t *)NAME, (uint8_t *)VALUE, sizeof(NAME) - 1, strlen(VALUE),        \
-        NGHTTP2_NV_FLAG_NONE                                                   \
+    (uint8_t *)NAME, (uint8_t *)VALUE,     sizeof(NAME) - 1,                   \
+    strlen(VALUE),   NGHTTP2_NV_FLAG_NONE,                                     \
   }
 
 struct Connection {
@@ -154,13 +155,14 @@ static void diec(const char *func, int error_code) {
 }
 
 /*
- * The implementation of nghttp2_send_callback type. Here we write
+ * The implementation of nghttp2_send_callback2 type. Here we write
  * |data| with size |length| to the network and return the number of
  * bytes actually written. See the documentation of
  * nghttp2_send_callback for the details.
  */
-static ssize_t send_callback(nghttp2_session *session, const uint8_t *data,
-                             size_t length, int flags, void *user_data) {
+static nghttp2_ssize send_callback(nghttp2_session *session,
+                                   const uint8_t *data, size_t length,
+                                   int flags, void *user_data) {
   struct Connection *connection;
   int rv;
   (void)session;
@@ -174,7 +176,7 @@ static ssize_t send_callback(nghttp2_session *session, const uint8_t *data,
     int err = SSL_get_error(connection->ssl, rv);
     if (err == SSL_ERROR_WANT_WRITE || err == SSL_ERROR_WANT_READ) {
       connection->want_io =
-          (err == SSL_ERROR_WANT_READ ? WANT_READ : WANT_WRITE);
+        (err == SSL_ERROR_WANT_READ ? WANT_READ : WANT_WRITE);
       rv = NGHTTP2_ERR_WOULDBLOCK;
     } else {
       rv = NGHTTP2_ERR_CALLBACK_FAILURE;
@@ -184,13 +186,14 @@ static ssize_t send_callback(nghttp2_session *session, const uint8_t *data,
 }
 
 /*
- * The implementation of nghttp2_recv_callback type. Here we read data
- * from the network and write them in |buf|. The capacity of |buf| is
- * |length| bytes. Returns the number of bytes stored in |buf|. See
- * the documentation of nghttp2_recv_callback for the details.
+ * The implementation of nghttp2_recv_callback2 type. Here we read
+ * data from the network and write them in |buf|. The capacity of
+ * |buf| is |length| bytes. Returns the number of bytes stored in
+ * |buf|. See the documentation of nghttp2_recv_callback for the
+ * details.
  */
-static ssize_t recv_callback(nghttp2_session *session, uint8_t *buf,
-                             size_t length, int flags, void *user_data) {
+static nghttp2_ssize recv_callback(nghttp2_session *session, uint8_t *buf,
+                                   size_t length, int flags, void *user_data) {
   struct Connection *connection;
   int rv;
   (void)session;
@@ -204,7 +207,7 @@ static ssize_t recv_callback(nghttp2_session *session, uint8_t *buf,
     int err = SSL_get_error(connection->ssl, rv);
     if (err == SSL_ERROR_WANT_WRITE || err == SSL_ERROR_WANT_READ) {
       connection->want_io =
-          (err == SSL_ERROR_WANT_READ ? WANT_READ : WANT_WRITE);
+        (err == SSL_ERROR_WANT_READ ? WANT_READ : WANT_WRITE);
       rv = NGHTTP2_ERR_WOULDBLOCK;
     } else {
       rv = NGHTTP2_ERR_CALLBACK_FAILURE;
@@ -328,9 +331,9 @@ static int on_data_chunk_recv_callback(nghttp2_session *session, uint8_t flags,
  * recv_callback is also required.
  */
 static void setup_nghttp2_callbacks(nghttp2_session_callbacks *callbacks) {
-  nghttp2_session_callbacks_set_send_callback(callbacks, send_callback);
+  nghttp2_session_callbacks_set_send_callback2(callbacks, send_callback);
 
-  nghttp2_session_callbacks_set_recv_callback(callbacks, recv_callback);
+  nghttp2_session_callbacks_set_recv_callback2(callbacks, recv_callback);
 
   nghttp2_session_callbacks_set_on_frame_send_callback(callbacks,
                                                        on_frame_send_callback);
@@ -339,35 +342,12 @@ static void setup_nghttp2_callbacks(nghttp2_session_callbacks *callbacks) {
                                                        on_frame_recv_callback);
 
   nghttp2_session_callbacks_set_on_stream_close_callback(
-      callbacks, on_stream_close_callback);
+    callbacks, on_stream_close_callback);
 
   nghttp2_session_callbacks_set_on_data_chunk_recv_callback(
-      callbacks, on_data_chunk_recv_callback);
+    callbacks, on_data_chunk_recv_callback);
 }
 
-#ifndef OPENSSL_NO_NEXTPROTONEG
-/*
- * Callback function for TLS NPN. Since this program only supports
- * HTTP/2 protocol, if server does not offer HTTP/2 the nghttp2
- * library supports, we terminate program.
- */
-static int select_next_proto_cb(SSL *ssl, unsigned char **out,
-                                unsigned char *outlen, const unsigned char *in,
-                                unsigned int inlen, void *arg) {
-  int rv;
-  (void)ssl;
-  (void)arg;
-
-  /* nghttp2_select_next_protocol() selects HTTP/2 protocol the
-     nghttp2 library supports. */
-  rv = nghttp2_select_next_protocol(out, outlen, in, inlen);
-  if (rv <= 0) {
-    die("Server did not advertise HTTP/2 protocol");
-  }
-  return SSL_TLSEXT_ERR_OK;
-}
-#endif /* !OPENSSL_NO_NEXTPROTONEG */
-
 /*
  * Setup SSL/TLS context.
  */
@@ -376,10 +356,8 @@ static void init_ssl_ctx(SSL_CTX *ssl_ctx) {
   SSL_CTX_set_options(ssl_ctx, SSL_OP_ALL | SSL_OP_NO_SSLv2);
   SSL_CTX_set_mode(ssl_ctx, SSL_MODE_AUTO_RETRY);
   SSL_CTX_set_mode(ssl_ctx, SSL_MODE_RELEASE_BUFFERS);
-  /* Set NPN callback */
-#ifndef OPENSSL_NO_NEXTPROTONEG
-  SSL_CTX_set_next_proto_select_cb(ssl_ctx, select_next_proto_cb, NULL);
-#endif /* !OPENSSL_NO_NEXTPROTONEG */
+
+  SSL_CTX_set_alpn_protos(ssl_ctx, (const unsigned char *)"\x02h2", 3);
 }
 
 static void ssl_handshake(SSL *ssl, int fd) {
@@ -483,8 +461,8 @@ static void submit_request(struct Connection *connection, struct Request *req) {
                             MAKE_NV("accept", "*/*"),
                             MAKE_NV("user-agent", "nghttp2/" NGHTTP2_VERSION)};
 
-  stream_id = nghttp2_submit_request(connection->session, NULL, nva,
-                                     sizeof(nva) / sizeof(nva[0]), NULL, req);
+  stream_id = nghttp2_submit_request2(connection->session, NULL, nva,
+                                      sizeof(nva) / sizeof(nva[0]), NULL, req);
 
   if (stream_id < 0) {
     diec("nghttp2_submit_request", stream_id);
@@ -544,7 +522,7 @@ static void fetch_uri(const struct URI *uri) {
   if (fd == -1) {
     die("Could not open file descriptor");
   }
-  ssl_ctx = SSL_CTX_new(SSLv23_client_method());
+  ssl_ctx = SSL_CTX_new(TLS_client_method());
   if (ssl_ctx == NULL) {
     dief("SSL_CTX_new", ERR_error_string(ERR_get_error(), NULL));
   }
@@ -715,9 +693,6 @@ int main(int argc, char **argv) {
   act.sa_handler = SIG_IGN;
   sigaction(SIGPIPE, &act, 0);
 
-  SSL_load_error_strings();
-  SSL_library_init();
-
   rv = parse_uri(&uri, argv[1]);
   if (rv != 0) {
     die("parse_uri failed");
