@@ -27,7 +27,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif // HAVE_CONFIG_H
+#endif // defined(HAVE_CONFIG_H)
 
 #define MUNIT_ENABLE_ASSERT_ALIASES
 
@@ -37,8 +37,8 @@ namespace nghttp2 {
 
 extern const MunitSuite buffer_suite;
 
-munit_void_test_decl(test_buffer_write);
+munit_void_test_decl(test_buffer_write)
 
 } // namespace nghttp2
 
-#endif // BUFFER_TEST_H
+#endif // !defined(BUFFER_TEST_H)
