@@ -34,9 +34,9 @@ struct iovec {
   void *iov_base; /* Pointer to data.  */
   size_t iov_len; /* Length of data.  */
 };
-#else // !_WIN32
-#include <sys/uio.h>
-#endif // !_WIN32
+#else // !defined(_WIN32)
+#  include <sys/uio.h>
+#endif // !defined(_WIN32)
 
 #include <cassert>
 #include <cstring>
@@ -44,6 +44,7 @@ struct iovec {
 #include <array>
 #include <algorithm>
 #include <string>
+#include <utility>
 
 #include "template.h"
 
@@ -52,17 +53,22 @@ namespace nghttp2 {
 #define DEFAULT_WR_IOVCNT 16
 
 #if defined(IOV_MAX) && IOV_MAX < DEFAULT_WR_IOVCNT
-#define MAX_WR_IOVCNT IOV_MAX
+#  define MAX_WR_IOVCNT IOV_MAX
 #else // !defined(IOV_MAX) || IOV_MAX >= DEFAULT_WR_IOVCNT
-#define MAX_WR_IOVCNT DEFAULT_WR_IOVCNT
+#  define MAX_WR_IOVCNT DEFAULT_WR_IOVCNT
 #endif // !defined(IOV_MAX) || IOV_MAX >= DEFAULT_WR_IOVCNT
 
 template <size_t N> struct Memchunk {
   Memchunk(Memchunk *next_chunk)
-      : pos(std::begin(buf)), last(pos), knext(next_chunk), next(nullptr) {}
-  size_t len() const { return last - pos; }
-  size_t left() const { return std::end(buf) - last; }
-  void reset() { pos = last = std::begin(buf); }
+    : pos(std::ranges::begin(buf)),
+      last(pos),
+      knext(next_chunk),
+      next(nullptr) {}
+  size_t len() const { return as_unsigned(last - pos); }
+  size_t left() const {
+    return static_cast<size_t>(std::ranges::end(buf) - last);
+  }
+  void reset() { pos = last = std::ranges::begin(buf); }
   std::array<uint8_t, N> buf;
   uint8_t *pos, *last;
   Memchunk *knext;
@@ -71,7 +77,7 @@ template <size_t N> struct Memchunk {
 };
 
 template <typename T> struct Pool {
-  Pool() : pool(nullptr), freelist(nullptr), poolsize(0) {}
+  Pool() : pool(nullptr), freelist(nullptr), poolsize(0), freelistsize(0) {}
   ~Pool() { clear(); }
   T *get() {
     if (freelist) {
@@ -79,6 +85,7 @@ template <typename T> struct Pool {
       freelist = freelist->next;
       m->next = nullptr;
       m->reset();
+      freelistsize -= T::size;
       return m;
     }
 
@@ -89,9 +96,11 @@ template <typename T> struct Pool {
   void recycle(T *m) {
     m->next = freelist;
     freelist = m;
+    freelistsize += T::size;
   }
   void clear() {
     freelist = nullptr;
+    freelistsize = 0;
     for (auto p = pool; p;) {
       auto knext = p->knext;
       delete p;
@@ -104,18 +113,27 @@ template <typename T> struct Pool {
   T *pool;
   T *freelist;
   size_t poolsize;
+  size_t freelistsize;
 };
 
 template <typename Memchunk> struct Memchunks {
   Memchunks(Pool<Memchunk> *pool)
-      : pool(pool), head(nullptr), tail(nullptr), len(0) {}
+    : pool(pool),
+      head(nullptr),
+      tail(nullptr),
+      len(0),
+      mark(nullptr),
+      mark_pos(nullptr),
+      mark_offset(0) {}
   Memchunks(const Memchunks &) = delete;
   Memchunks(Memchunks &&other) noexcept
-      : pool(other.pool), head(other.head), tail(other.tail), len(other.len) {
-    // keep other.pool
-    other.head = other.tail = nullptr;
-    other.len = 0;
-  }
+    : pool{other.pool}, // keep other.pool
+      head{std::exchange(other.head, nullptr)},
+      tail{std::exchange(other.tail, nullptr)},
+      len{std::exchange(other.len, 0)},
+      mark{std::exchange(other.mark, nullptr)},
+      mark_pos{std::exchange(other.mark_pos, nullptr)},
+      mark_offset{std::exchange(other.mark_offset, 0)} {}
   Memchunks &operator=(const Memchunks &) = delete;
   Memchunks &operator=(Memchunks &&other) noexcept {
     if (this == &other) {
@@ -125,12 +143,12 @@ template <typename Memchunk> struct Memchunks {
     reset();
 
     pool = other.pool;
-    head = other.head;
-    tail = other.tail;
-    len = other.len;
-
-    other.head = other.tail = nullptr;
-    other.len = 0;
+    head = std::exchange(other.head, nullptr);
+    tail = std::exchange(other.tail, nullptr);
+    len = std::exchange(other.len, 0);
+    mark = std::exchange(other.mark, nullptr);
+    mark_pos = std::exchange(other.mark_pos, nullptr);
+    mark_offset = std::exchange(other.mark_offset, 0);
 
     return *this;
   }
@@ -144,33 +162,31 @@ template <typename Memchunk> struct Memchunks {
       m = next;
     }
   }
-  size_t append(char c) {
+  void append(char c) {
     if (!tail) {
       head = tail = pool->get();
     } else if (tail->left() == 0) {
       tail->next = pool->get();
       tail = tail->next;
     }
-    *tail->last++ = c;
+    *tail->last++ = as_unsigned(c);
     ++len;
-    return 1;
   }
-  size_t append(const void *src, size_t count) {
-    if (count == 0) {
-      return 0;
+  template <std::input_iterator I> void append(I first, I last) {
+    if (first == last) {
+      return;
     }
 
-    auto first = static_cast<const uint8_t *>(src);
-    auto last = first + count;
-
     if (!tail) {
       head = tail = pool->get();
     }
 
     for (;;) {
-      auto n = std::min(static_cast<size_t>(last - first), tail->left());
-      tail->last = std::copy_n(first, n, tail->last);
-      first += n;
+      auto n = std::min(static_cast<size_t>(std::ranges::distance(first, last)),
+                        tail->left());
+      auto iores = std::ranges::copy_n(first, as_signed(n), tail->last);
+      first = iores.in;
+      tail->last = iores.out;
       len += n;
       if (first == last) {
         break;
@@ -180,17 +196,51 @@ template <typename Memchunk> struct Memchunks {
       tail = tail->next;
     }
 
-    return count;
+    return;
   }
-  template <size_t N> size_t append(const char (&s)[N]) {
-    return append(s, N - 1);
+  void append(const void *src, size_t count) {
+    auto s = static_cast<const uint8_t *>(src);
+    append(s, s + count);
   }
-  size_t append(const std::string &s) { return append(s.c_str(), s.size()); }
-  size_t append(const StringRef &s) { return append(s.c_str(), s.size()); }
-  size_t append(const ImmutableString &s) {
-    return append(s.c_str(), s.size());
+  template <std::ranges::input_range R>
+  requires(!std::is_array_v<std::remove_cvref_t<R>>)
+  void append(R &&r) {
+    append(std::ranges::begin(r), std::ranges::end(r));
+  }
+  // first ensures that at least |max_count| bytes are available to
+  // store in the current buffer, assuming that the chunk size of the
+  // underlying Memchunk is at least |max_count| bytes.  Then call
+  // |f|(tail->last) to write data into buffer directly.  |f| must not
+  // write more than |max_count| bytes.  It must return the position
+  // of the buffer past the last position written.
+  template <typename F>
+  requires(std::invocable<F &, uint8_t *> &&
+           std::is_same_v<std::invoke_result_t<F &, uint8_t *>, uint8_t *>)
+  void append(size_t max_count, F f) {
+    if (!tail) {
+      head = tail = pool->get();
+    } else if (tail->left() < max_count) {
+      tail->next = pool->get();
+      tail = tail->next;
+    }
+
+    assert(tail->left() >= max_count);
+
+    auto last = f(tail->last);
+    len += static_cast<size_t>(last - tail->last);
+    tail->last = last;
+  }
+  size_t copy(Memchunks &dest) {
+    auto m = head;
+    while (m) {
+      dest.append(m->pos, m->len());
+      m = m->next;
+    }
+    return len;
   }
   size_t remove(void *dest, size_t count) {
+    assert(mark == nullptr);
+
     if (!tail || count == 0) {
       return 0;
     }
@@ -205,8 +255,9 @@ template <typename Memchunk> struct Memchunks {
       auto n = std::min(static_cast<size_t>(last - first), m->len());
 
       assert(m->len());
-      first = std::copy_n(m->pos, n, first);
-      m->pos += n;
+      auto iores = std::ranges::copy_n(m->pos, as_signed(n), first);
+      m->pos = iores.in;
+      first = iores.out;
       len -= n;
       if (m->len() > 0) {
         break;
@@ -219,9 +270,11 @@ template <typename Memchunk> struct Memchunks {
       tail = nullptr;
     }
 
-    return first - static_cast<uint8_t *>(dest);
+    return as_unsigned(first - static_cast<uint8_t *>(dest));
   }
   size_t remove(Memchunks &dest, size_t count) {
+    assert(mark == nullptr);
+
     if (!tail || count == 0) {
       return 0;
     }
@@ -251,7 +304,33 @@ template <typename Memchunk> struct Memchunks {
 
     return count - left;
   }
+  size_t remove(Memchunks &dest) {
+    assert(pool == dest.pool);
+    assert(mark == nullptr);
+
+    if (head == nullptr) {
+      return 0;
+    }
+
+    auto n = len;
+
+    if (dest.tail == nullptr) {
+      dest.head = head;
+    } else {
+      dest.tail->next = head;
+    }
+
+    dest.tail = tail;
+    dest.len += len;
+
+    head = tail = nullptr;
+    len = 0;
+
+    return n;
+  }
   size_t drain(size_t count) {
+    assert(mark == nullptr);
+
     auto ndata = count;
     auto m = head;
     while (m) {
@@ -273,6 +352,38 @@ template <typename Memchunk> struct Memchunks {
     }
     return ndata - count;
   }
+  size_t drain_mark(size_t count) {
+    auto ndata = count;
+    auto m = head;
+    while (m) {
+      auto next = m->next;
+      auto n = std::min(count, m->len());
+      m->pos += n;
+      count -= n;
+      len -= n;
+      mark_offset -= n;
+
+      if (m->len() > 0) {
+        assert(mark != m || m->pos <= mark_pos);
+        break;
+      }
+      if (mark == m) {
+        assert(m->pos <= mark_pos);
+
+        mark = nullptr;
+        mark_pos = nullptr;
+        mark_offset = 0;
+      }
+
+      pool->recycle(m);
+      m = next;
+    }
+    head = m;
+    if (head == nullptr) {
+      tail = nullptr;
+    }
+    return ndata - count;
+  }
   int riovec(struct iovec *iov, int iovcnt) const {
     if (!head) {
       return 0;
@@ -285,7 +396,41 @@ template <typename Memchunk> struct Memchunks {
     }
     return i;
   }
+  int riovec_mark(struct iovec *iov, int iovcnt) {
+    if (!head || iovcnt == 0) {
+      return 0;
+    }
+
+    int i = 0;
+    Memchunk *m;
+    if (mark) {
+      if (mark_pos != mark->last) {
+        iov[0].iov_base = mark_pos;
+        iov[0].iov_len = mark->len() - as_unsigned(mark_pos - mark->pos);
+
+        mark_pos = mark->last;
+        mark_offset += iov[0].iov_len;
+        i = 1;
+      }
+      m = mark->next;
+    } else {
+      i = 0;
+      m = head;
+    }
+
+    for (; i < iovcnt && m; ++i, m = m->next) {
+      iov[i].iov_base = m->pos;
+      iov[i].iov_len = m->len();
+
+      mark = m;
+      mark_pos = m->last;
+      mark_offset += m->len();
+    }
+
+    return i;
+  }
   size_t rleft() const { return len; }
+  size_t rleft_mark() const { return len - mark_offset; }
   void reset() {
     for (auto m = head; m;) {
       auto next = m->next;
@@ -293,142 +438,22 @@ template <typename Memchunk> struct Memchunks {
       m = next;
     }
     len = 0;
-    head = tail = nullptr;
+    head = tail = mark = nullptr;
+    mark_pos = nullptr;
+    mark_offset = 0;
   }
 
   Pool<Memchunk> *pool;
   Memchunk *head, *tail;
   size_t len;
-};
-
-// Wrapper around Memchunks to offer "peeking" functionality.
-template <typename Memchunk> struct PeekMemchunks {
-  PeekMemchunks(Pool<Memchunk> *pool)
-      : memchunks(pool),
-        cur(nullptr),
-        cur_pos(nullptr),
-        cur_last(nullptr),
-        len(0),
-        peeking(true) {}
-  PeekMemchunks(const PeekMemchunks &) = delete;
-  PeekMemchunks(PeekMemchunks &&other) noexcept
-      : memchunks(std::move(other.memchunks)),
-        cur(other.cur),
-        cur_pos(other.cur_pos),
-        cur_last(other.cur_last),
-        len(other.len),
-        peeking(other.peeking) {
-    other.reset();
-  }
-  PeekMemchunks &operator=(const PeekMemchunks &) = delete;
-  PeekMemchunks &operator=(PeekMemchunks &&other) noexcept {
-    if (this == &other) {
-      return *this;
-    }
-
-    memchunks = std::move(other.memchunks);
-    cur = other.cur;
-    cur_pos = other.cur_pos;
-    cur_last = other.cur_last;
-    len = other.len;
-    peeking = other.peeking;
-
-    other.reset();
-
-    return *this;
-  }
-  size_t append(const void *src, size_t count) {
-    count = memchunks.append(src, count);
-    len += count;
-    return count;
-  }
-  size_t remove(void *dest, size_t count) {
-    if (!peeking) {
-      count = memchunks.remove(dest, count);
-      len -= count;
-      return count;
-    }
-
-    if (count == 0 || len == 0) {
-      return 0;
-    }
-
-    if (!cur) {
-      cur = memchunks.head;
-      cur_pos = cur->pos;
-    }
-
-    // cur_last could be updated in append
-    cur_last = cur->last;
-
-    if (cur_pos == cur_last) {
-      assert(cur->next);
-      cur = cur->next;
-    }
-
-    auto first = static_cast<uint8_t *>(dest);
-    auto last = first + count;
-
-    for (;;) {
-      auto n = std::min(last - first, cur_last - cur_pos);
-
-      first = std::copy_n(cur_pos, n, first);
-      cur_pos += n;
-      len -= n;
-
-      if (first == last) {
-        break;
-      }
-      assert(cur_pos == cur_last);
-      if (!cur->next) {
-        break;
-      }
-      cur = cur->next;
-      cur_pos = cur->pos;
-      cur_last = cur->last;
-    }
-    return first - static_cast<uint8_t *>(dest);
-  }
-  size_t rleft() const { return len; }
-  size_t rleft_buffered() const { return memchunks.rleft(); }
-  void disable_peek(bool drain) {
-    if (!peeking) {
-      return;
-    }
-    if (drain) {
-      auto n = rleft_buffered() - rleft();
-      memchunks.drain(n);
-      assert(len == memchunks.rleft());
-    } else {
-      len = memchunks.rleft();
-    }
-    cur = nullptr;
-    cur_pos = cur_last = nullptr;
-    peeking = false;
-  }
-  void reset() {
-    memchunks.reset();
-    cur = nullptr;
-    cur_pos = cur_last = nullptr;
-    len = 0;
-    peeking = true;
-  }
-  Memchunks<Memchunk> memchunks;
-  // Pointer to the Memchunk currently we are reading/writing.
-  Memchunk *cur;
-  // Region inside cur, we have processed to cur_pos.
-  uint8_t *cur_pos, *cur_last;
-  // This is the length we have left unprocessed.  len <=
-  // memchunk.rleft() must hold.
-  size_t len;
-  // true if peeking is enabled.  Initially it is true.
-  bool peeking;
+  Memchunk *mark;
+  uint8_t *mark_pos;
+  size_t mark_offset;
 };
 
 using Memchunk16K = Memchunk<16_k>;
 using MemchunkPool = Pool<Memchunk16K>;
 using DefaultMemchunks = Memchunks<Memchunk16K>;
-using DefaultPeekMemchunks = PeekMemchunks<Memchunk16K>;
 
 inline int limit_iovec(struct iovec *iov, int iovcnt, size_t max) {
   if (max == 0) {
@@ -451,7 +476,7 @@ template <typename Memchunk> struct MemchunkBuffer {
   MemchunkBuffer(Pool<Memchunk> *pool) : pool(pool), chunk(nullptr) {}
   MemchunkBuffer(const MemchunkBuffer &) = delete;
   MemchunkBuffer(MemchunkBuffer &&other) noexcept
-      : pool(other.pool), chunk(other.chunk) {
+    : pool(other.pool), chunk(other.chunk) {
     other.chunk = nullptr;
   }
   MemchunkBuffer &operator=(const MemchunkBuffer &) = delete;
@@ -510,7 +535,7 @@ template <typename Memchunk> struct MemchunkBuffer {
   size_t write(const void *src, size_t count) {
     count = std::min(count, wleft());
     auto p = static_cast<const uint8_t *>(src);
-    chunk->last = std::copy_n(p, count, chunk->last);
+    chunk->last = std::ranges::copy_n(p, count, chunk->last).out;
     return count;
   }
   size_t write(size_t count) {
@@ -525,13 +550,14 @@ template <typename Memchunk> struct MemchunkBuffer {
   }
   size_t drain_reset(size_t count) {
     count = std::min(count, rleft());
-    std::copy(chunk->pos + count, chunk->last, std::begin(chunk->buf));
-    chunk->last = std::begin(chunk->buf) + (chunk->last - (chunk->pos + count));
-    chunk->pos = std::begin(chunk->buf);
+    chunk->last = std::ranges::copy(chunk->pos + count, chunk->last,
+                                    std::ranges::begin(chunk->buf))
+                    .out;
+    chunk->pos = std::ranges::begin(chunk->buf);
     return count;
   }
   void reset() { chunk->reset(); }
-  uint8_t *begin() { return std::begin(chunk->buf); }
+  uint8_t *begin() { return std::ranges::begin(chunk->buf); }
   uint8_t &operator[](size_t n) { return chunk->buf[n]; }
   const uint8_t &operator[](size_t n) const { return chunk->buf[n]; }
 
@@ -543,4 +569,4 @@ using DefaultMemchunkBuffer = MemchunkBuffer<Memchunk16K>;
 
 } // namespace nghttp2
 
-#endif // MEMCHUNK_H
+#endif // !defined(MEMCHUNK_H)
