@@ -43,7 +43,7 @@
 #include "shrpx_connection_handler.h"
 #ifdef HAVE_MRUBY
 #  include "shrpx_mruby.h"
-#endif // HAVE_MRUBY
+#endif // defined(HAVE_MRUBY)
 #include "http3.h"
 #include "util.h"
 
@@ -53,10 +53,12 @@ namespace {
 void timeoutcb(struct ev_loop *loop, ev_timer *w, int revents) {
   auto upstream = static_cast<Http3Upstream *>(w->data);
 
-  if (upstream->handle_expiry() != 0 || upstream->on_write() != 0) {
+  if (upstream->handle_expiry() != 0) {
     goto fail;
   }
 
+  upstream->get_client_handler()->signal_write();
+
   return;
 
 fail:
@@ -109,6 +111,10 @@ ngtcp2_conn *get_conn(ngtcp2_crypto_conn_ref *conn_ref) {
 }
 } // namespace
 
+namespace {
+constexpr size_t QUIC_TX_DATALEN = 64_k;
+} // namespace
+
 Http3Upstream::Http3Upstream(ClientHandler *handler)
   : handler_{handler},
     qlog_fd_{-1},
@@ -121,10 +127,10 @@ Http3Upstream::Http3Upstream(ClientHandler *handler)
     downstream_queue_{downstream_queue_size(handler->get_worker()),
                       !get_config()->http2_proxy},
     tx_{
-      .data = std::unique_ptr<uint8_t[]>(new uint8_t[64_k]),
+      .data = std::unique_ptr<uint8_t[]>(new uint8_t[QUIC_TX_DATALEN]),
 #ifndef UDP_SEGMENT
       .no_gso = true,
-#endif // UDP_SEGMENT
+#endif // !defined(UDP_SEGMENT)
     } {
   auto conn = handler_->get_connection();
   conn->conn_ref.get_conn = shrpx::get_conn;
@@ -205,7 +211,7 @@ void Http3Upstream::qlog_write(const void *data, size_t datalen, bool fin) {
 }
 
 namespace {
-void rand(uint8_t *dest, size_t destlen, const ngtcp2_rand_ctx *rand_ctx) {
+void rand_bytes(uint8_t *dest, size_t destlen) {
   auto rv =
     RAND_bytes(dest, static_cast<nghttp2_ssl_rand_length_type>(destlen));
   if (rv != 1) {
@@ -215,6 +221,12 @@ void rand(uint8_t *dest, size_t destlen, const ngtcp2_rand_ctx *rand_ctx) {
 }
 } // namespace
 
+namespace {
+void rand(uint8_t *dest, size_t destlen, const ngtcp2_rand_ctx *rand_ctx) {
+  rand_bytes(dest, destlen);
+}
+} // namespace
+
 namespace {
 int get_new_connection_id(ngtcp2_conn *conn, ngtcp2_cid *cid, uint8_t *token,
                           size_t cidlen, void *user_data) {
@@ -299,11 +311,11 @@ int Http3Upstream::recv_stream_data(uint32_t flags, int64_t stream_id,
                                     std::span<const uint8_t> data) {
   assert(httpconn_);
 
-  auto nconsumed =
-    nghttp3_conn_read_stream(httpconn_, stream_id, data.data(), data.size(),
-                             flags & NGTCP2_STREAM_DATA_FLAG_FIN);
+  auto nconsumed = nghttp3_conn_read_stream2(
+    httpconn_, stream_id, data.data(), data.size(),
+    flags & NGTCP2_STREAM_DATA_FLAG_FIN, ngtcp2_conn_get_timestamp(conn_));
   if (nconsumed < 0) {
-    ULOG(ERROR, this) << "nghttp3_conn_read_stream: "
+    ULOG(ERROR, this) << "nghttp3_conn_read_stream2: "
                       << nghttp3_strerror(static_cast<int>(nconsumed));
     ngtcp2_ccerr_set_application_error(
       &last_error_,
@@ -575,45 +587,28 @@ int Http3Upstream::init(const UpstreamAddr *faddr, const Address &remote_addr,
   auto conn_handler = worker->get_connection_handler();
 
   auto callbacks = ngtcp2_callbacks{
-    nullptr, // client_initial
-    ngtcp2_crypto_recv_client_initial_cb,
-    ngtcp2_crypto_recv_crypto_data_cb,
-    shrpx::handshake_completed,
-    nullptr, // recv_version_negotiation
-    ngtcp2_crypto_encrypt_cb,
-    ngtcp2_crypto_decrypt_cb,
-    ngtcp2_crypto_hp_mask_cb,
-    shrpx::recv_stream_data,
-    shrpx::acked_stream_data_offset,
-    nullptr, // stream_open
-    shrpx::stream_close,
-    nullptr, // recv_stateless_reset
-    nullptr, // recv_retry
-    nullptr, // extend_max_local_streams_bidi
-    nullptr, // extend_max_local_streams_uni
-    rand,
-    get_new_connection_id,
-    remove_connection_id,
-    ngtcp2_crypto_update_key_cb,
-    shrpx::path_validation,
-    nullptr, // select_preferred_addr
-    shrpx::stream_reset,
-    shrpx::extend_max_remote_streams_bidi,
-    nullptr, // extend_max_remote_streams_uni
-    shrpx::extend_max_stream_data,
-    nullptr, // dcid_status
-    nullptr, // handshake_confirmed
-    nullptr, // recv_new_token
-    ngtcp2_crypto_delete_crypto_aead_ctx_cb,
-    ngtcp2_crypto_delete_crypto_cipher_ctx_cb,
-    nullptr, // recv_datagram
-    nullptr, // ack_datagram
-    nullptr, // lost_datagram
-    ngtcp2_crypto_get_path_challenge_data_cb,
-    shrpx::stream_stop_sending,
-    nullptr, // version_negotiation
-    nullptr, // recv_rx_key
-    shrpx::recv_tx_key,
+    .recv_client_initial = ngtcp2_crypto_recv_client_initial_cb,
+    .recv_crypto_data = ngtcp2_crypto_recv_crypto_data_cb,
+    .handshake_completed = shrpx::handshake_completed,
+    .encrypt = ngtcp2_crypto_encrypt_cb,
+    .decrypt = ngtcp2_crypto_decrypt_cb,
+    .hp_mask = ngtcp2_crypto_hp_mask_cb,
+    .recv_stream_data = shrpx::recv_stream_data,
+    .acked_stream_data_offset = shrpx::acked_stream_data_offset,
+    .stream_close = shrpx::stream_close,
+    .rand = rand,
+    .get_new_connection_id = get_new_connection_id,
+    .remove_connection_id = remove_connection_id,
+    .update_key = ngtcp2_crypto_update_key_cb,
+    .path_validation = shrpx::path_validation,
+    .stream_reset = shrpx::stream_reset,
+    .extend_max_remote_streams_bidi = shrpx::extend_max_remote_streams_bidi,
+    .extend_max_stream_data = shrpx::extend_max_stream_data,
+    .delete_crypto_aead_ctx = ngtcp2_crypto_delete_crypto_aead_ctx_cb,
+    .delete_crypto_cipher_ctx = ngtcp2_crypto_delete_crypto_cipher_ctx_cb,
+    .get_path_challenge_data = ngtcp2_crypto_get_path_challenge_data_cb,
+    .stream_stop_sending = shrpx::stream_stop_sending,
+    .recv_tx_key = shrpx::recv_tx_key,
   };
 
   auto config = get_config();
@@ -709,7 +704,7 @@ int Http3Upstream::init(const UpstreamAddr *faddr, const Address &remote_addr,
       return -1;
     }
   }
-#endif // NGHTTP2_OPENSSL_IS_BORINGSSL
+#endif // defined(NGHTTP2_OPENSSL_IS_BORINGSSL)
 
   if (odcid) {
     params.original_dcid = *odcid;
@@ -814,25 +809,21 @@ int Http3Upstream::on_write() {
   return 0;
 }
 
-int Http3Upstream::write_streams() {
-  std::array<nghttp3_vec, 16> vec;
-  auto max_udp_payload_size = ngtcp2_conn_get_max_tx_udp_payload_size(conn_);
-  auto path_max_udp_payload_size =
-    ngtcp2_conn_get_path_max_tx_udp_payload_size(conn_);
-  ngtcp2_pkt_info pi, prev_pi;
-  auto txbuf =
-    std::span{tx_.data.get(), std::max(ngtcp2_conn_get_send_quantum(conn_),
-                                       path_max_udp_payload_size)};
-  auto buf = txbuf;
-  auto pkt = std::span<const uint8_t>{};
-  auto extra_pkt = std::span<const uint8_t>{};
-  ngtcp2_path_storage ps, prev_ps;
-  int rv;
-  size_t gso_size = 0;
-  auto ts = quic_timestamp();
+namespace {
+ngtcp2_ssize write_pkt(ngtcp2_conn *conn, ngtcp2_path *path,
+                       ngtcp2_pkt_info *pi, uint8_t *dest, size_t destlen,
+                       ngtcp2_tstamp ts, void *user_data) {
+  auto upstream = static_cast<Http3Upstream *>(user_data);
 
-  ngtcp2_path_storage_zero(&ps);
-  ngtcp2_path_storage_zero(&prev_ps);
+  return upstream->write_pkt(path, pi, dest, destlen, ts);
+}
+} // namespace
+
+ngtcp2_ssize Http3Upstream::write_pkt(ngtcp2_path *path, ngtcp2_pkt_info *pi,
+                                      uint8_t *dest, size_t destlen,
+                                      ngtcp2_tstamp ts) {
+  std::array<nghttp3_vec, 16> vec;
+  int rv;
 
   for (;;) {
     int64_t stream_id = -1;
@@ -849,7 +840,7 @@ int Http3Upstream::write_streams() {
           &last_error_,
           nghttp3_err_infer_quic_app_error_code(static_cast<int>(sveccnt)),
           nullptr, 0);
-        return handle_error();
+        return NGTCP2_ERR_CALLBACK_FAILURE;
       }
     }
 
@@ -857,16 +848,14 @@ int Http3Upstream::write_streams() {
     auto v = vec.data();
     auto vcnt = static_cast<size_t>(sveccnt);
 
-    uint32_t flags = NGTCP2_WRITE_STREAM_FLAG_MORE;
+    uint32_t flags =
+      NGTCP2_WRITE_STREAM_FLAG_MORE | NGTCP2_WRITE_STREAM_FLAG_PADDING;
     if (fin) {
       flags |= NGTCP2_WRITE_STREAM_FLAG_FIN;
     }
 
-    auto buflen = buf.size() >= max_udp_payload_size
-                    ? max_udp_payload_size
-                    : path_max_udp_payload_size;
     auto nwrite = ngtcp2_conn_writev_stream(
-      conn_, &ps.path, &pi, buf.data(), buflen, &ndatalen, flags, stream_id,
+      conn_, path, pi, dest, destlen, &ndatalen, flags, stream_id,
       reinterpret_cast<const ngtcp2_vec *>(v), vcnt, ts);
     if (nwrite < 0) {
       switch (nwrite) {
@@ -888,7 +877,7 @@ int Http3Upstream::write_streams() {
           ngtcp2_ccerr_set_application_error(
             &last_error_, nghttp3_err_infer_quic_app_error_code(rv), nullptr,
             0);
-          return handle_error();
+          return NGTCP2_ERR_CALLBACK_FAILURE;
         }
         continue;
       }
@@ -901,8 +890,10 @@ int Http3Upstream::write_streams() {
       ngtcp2_ccerr_set_liberr(&last_error_, static_cast<int>(nwrite), nullptr,
                               0);
 
-      return handle_error();
-    } else if (ndatalen >= 0) {
+      return NGTCP2_ERR_CALLBACK_FAILURE;
+    }
+
+    if (ndatalen >= 0) {
       rv = nghttp3_conn_add_write_offset(httpconn_, stream_id,
                                          as_unsigned(ndatalen));
       if (rv != 0) {
@@ -910,64 +901,42 @@ int Http3Upstream::write_streams() {
                           << nghttp3_strerror(rv);
         ngtcp2_ccerr_set_application_error(
           &last_error_, nghttp3_err_infer_quic_app_error_code(rv), nullptr, 0);
-        return handle_error();
+        return NGTCP2_ERR_CALLBACK_FAILURE;
       }
     }
 
-    if (nwrite == 0) {
-      pkt = std::span{std::ranges::begin(txbuf), std::ranges::begin(buf)};
-      if (pkt.empty()) {
-        return 0;
-      }
+    return nwrite;
+  }
+}
 
-      break;
-    }
+int Http3Upstream::write_streams() {
+  ngtcp2_path_storage ps;
+  ngtcp2_pkt_info pi;
+  auto txbuf = std::span{tx_.data.get(), QUIC_TX_DATALEN};
+  size_t gso_size = 0;
 
-    auto last_pkt_pos = std::ranges::begin(buf);
+  ngtcp2_path_storage_zero(&ps);
 
-    buf = buf.subspan(as_unsigned(nwrite));
-
-    if (last_pkt_pos == std::ranges::begin(txbuf)) {
-      ngtcp2_path_copy(&prev_ps.path, &ps.path);
-      prev_pi = pi;
-      gso_size = as_unsigned(nwrite);
-    } else if (!ngtcp2_path_eq(&prev_ps.path, &ps.path) ||
-               prev_pi.ecn != pi.ecn ||
-               static_cast<size_t>(nwrite) > gso_size ||
-               (gso_size > path_max_udp_payload_size &&
-                static_cast<size_t>(nwrite) != gso_size)) {
-      pkt = std::span{std::ranges::begin(txbuf), last_pkt_pos};
-      extra_pkt = std::span{last_pkt_pos, std::ranges::begin(buf)};
-      break;
-    }
-
-    if (buf.size() < path_max_udp_payload_size ||
-        static_cast<size_t>(nwrite) < gso_size) {
-      pkt = std::span{std::ranges::begin(txbuf), std::ranges::begin(buf)};
-      break;
-    }
+  auto nwrite = ngtcp2_conn_write_aggregate_pkt(
+    conn_, &ps.path, &pi, txbuf.data(), txbuf.size(), &gso_size,
+    shrpx::write_pkt, quic_timestamp());
+  if (nwrite < 0) {
+    return handle_error();
   }
 
-  assert(!pkt.empty());
-
-  if (send_packet(prev_ps.path, prev_pi, pkt, gso_size) ==
-      SHRPX_ERR_SEND_BLOCKED) {
-    if (!extra_pkt.empty()) {
-      on_send_blocked(ps.path, pi, extra_pkt, extra_pkt.size());
-    }
-  } else if (!extra_pkt.empty()) {
-    send_packet(ps.path, pi, extra_pkt, extra_pkt.size());
+  if (nwrite == 0) {
+    return 0;
   }
 
-  ngtcp2_conn_update_pkt_tx_time(conn_, ts);
+  send_packet(ps.path, pi, txbuf.first(static_cast<size_t>(nwrite)), gso_size);
 
   return 0;
 }
 
-int Http3Upstream::send_packet(const ngtcp2_path &path,
-                               const ngtcp2_pkt_info &pi,
-                               const std::span<const uint8_t> data,
-                               size_t gso_size) {
+void Http3Upstream::send_packet(const ngtcp2_path &path,
+                                const ngtcp2_pkt_info &pi,
+                                const std::span<const uint8_t> data,
+                                size_t gso_size) {
   auto faddr = static_cast<UpstreamAddr *>(path.user_data);
 
   auto [rest, rv] =
@@ -978,8 +947,6 @@ int Http3Upstream::send_packet(const ngtcp2_path &path,
 
     signal_write_upstream_addr(faddr);
   }
-
-  return rv;
 }
 
 int Http3Upstream::on_timeout(Downstream *downstream) {
@@ -1303,7 +1270,7 @@ int Http3Upstream::on_downstream_header_complete(Downstream *downstream) {
       return -1;
     }
   }
-#endif // HAVE_MRUBY
+#endif // defined(HAVE_MRUBY)
 
   auto nva = std::vector<nghttp3_nv>();
   // 4 means :status and possible server, via, and set-cookie (for
@@ -1428,8 +1395,9 @@ int Http3Upstream::on_downstream_header_complete(Downstream *downstream) {
     }
   }
 
-  nghttp3_data_reader data_read;
-  data_read.read_data = downstream_read_data_callback;
+  nghttp3_data_reader data_read{
+    .read_data = downstream_read_data_callback,
+  };
 
   nghttp3_data_reader *data_readptr;
 
@@ -1931,13 +1899,12 @@ void Http3Upstream::on_send_blocked(const ngtcp2_path &path,
                                     const ngtcp2_pkt_info &pi,
                                     std::span<const uint8_t> data,
                                     size_t gso_size) {
-  assert(tx_.num_blocked || !tx_.send_blocked);
-  assert(tx_.num_blocked < 2);
+  assert(!tx_.send_blocked);
   assert(gso_size);
 
   tx_.send_blocked = true;
 
-  auto &p = tx_.blocked[tx_.num_blocked++];
+  auto &p = tx_.blocked;
 
   memcpy(&p.local_addr.su, path.local.addr, path.local.addrlen);
   memcpy(&p.remote_addr.su, path.remote.addr, path.remote.addrlen);
@@ -1953,24 +1920,20 @@ void Http3Upstream::on_send_blocked(const ngtcp2_path &path,
 int Http3Upstream::send_blocked_packet() {
   assert(tx_.send_blocked);
 
-  for (; tx_.num_blocked_sent < tx_.num_blocked; ++tx_.num_blocked_sent) {
-    auto &p = tx_.blocked[tx_.num_blocked_sent];
+  auto &p = tx_.blocked;
 
-    auto [rest, rv] = send_packet(p.faddr, &p.remote_addr.su.sa,
-                                  p.remote_addr.len, &p.local_addr.su.sa,
-                                  p.local_addr.len, p.pi, p.data, p.gso_size);
-    if (rv == SHRPX_ERR_SEND_BLOCKED) {
-      p.data = rest;
+  auto [rest, rv] = send_packet(p.faddr, &p.remote_addr.su.sa,
+                                p.remote_addr.len, &p.local_addr.su.sa,
+                                p.local_addr.len, p.pi, p.data, p.gso_size);
+  if (rv == SHRPX_ERR_SEND_BLOCKED) {
+    p.data = rest;
 
-      signal_write_upstream_addr(p.faddr);
+    signal_write_upstream_addr(p.faddr);
 
-      return 0;
-    }
+    return 0;
   }
 
   tx_.send_blocked = false;
-  tx_.num_blocked = 0;
-  tx_.num_blocked_sent = 0;
 
   return 0;
 }
@@ -2369,7 +2332,7 @@ int Http3Upstream::http_end_request_headers(Downstream *downstream, int fin) {
     }
     return 0;
   }
-#endif // HAVE_MRUBY
+#endif // defined(HAVE_MRUBY)
 
   if (downstream->get_response_state() == DownstreamState::MSG_COMPLETE) {
     return 0;
@@ -2394,7 +2357,7 @@ void Http3Upstream::initiate_downstream(Downstream *downstream) {
 
 #ifdef HAVE_MRUBY
   DownstreamConnection *dconn_ptr;
-#endif // HAVE_MRUBY
+#endif // defined(HAVE_MRUBY)
 
   for (;;) {
     auto dconn = handler_->get_downstream_connection(rv, downstream);
@@ -2417,7 +2380,7 @@ void Http3Upstream::initiate_downstream(Downstream *downstream) {
 
 #ifdef HAVE_MRUBY
     dconn_ptr = dconn.get();
-#endif // HAVE_MRUBY
+#endif // defined(HAVE_MRUBY)
     rv = downstream->attach_downstream_connection(std::move(dconn));
     if (rv == 0) {
       break;
@@ -2442,7 +2405,7 @@ void Http3Upstream::initiate_downstream(Downstream *downstream) {
       return;
     }
   }
-#endif // HAVE_MRUBY
+#endif // defined(HAVE_MRUBY)
 
   rv = downstream->push_request_headers();
   if (rv != 0) {
@@ -2657,19 +2620,18 @@ int Http3Upstream::setup_httpconn() {
   }
 
   nghttp3_callbacks callbacks{
-    shrpx::http_acked_stream_data,
-    shrpx::http_stream_close,
-    shrpx::http_recv_data,
-    http_deferred_consume,
-    shrpx::http_begin_request_headers,
-    shrpx::http_recv_request_header,
-    shrpx::http_end_request_headers,
-    nullptr, // begin_trailers
-    shrpx::http_recv_request_trailer,
-    nullptr, // end_trailers
-    shrpx::http_stop_sending,
-    shrpx::http_end_stream,
-    shrpx::http_reset_stream,
+    .acked_stream_data = shrpx::http_acked_stream_data,
+    .stream_close = shrpx::http_stream_close,
+    .recv_data = shrpx::http_recv_data,
+    .deferred_consume = http_deferred_consume,
+    .begin_headers = shrpx::http_begin_request_headers,
+    .recv_header = shrpx::http_recv_request_header,
+    .end_headers = shrpx::http_end_request_headers,
+    .recv_trailer = shrpx::http_recv_request_trailer,
+    .stop_sending = shrpx::http_stop_sending,
+    .end_stream = shrpx::http_end_stream,
+    .reset_stream = shrpx::http_reset_stream,
+    .rand = shrpx::rand_bytes,
   };
 
   auto config = get_config();
@@ -2935,7 +2897,7 @@ int Http3Upstream::open_qlog_file(const std::string_view &dir,
                     S_IRUSR | S_IWUSR | S_IRGRP)) == -1 &&
          errno == EINTR)
     ;
-#else  // !O_CLOEXEC
+#else  // !defined(O_CLOEXEC)
   while ((fd = open(path.c_str(), O_WRONLY | O_CREAT | O_TRUNC,
                     S_IRUSR | S_IWUSR | S_IRGRP)) == -1 &&
          errno == EINTR)
@@ -2944,7 +2906,7 @@ int Http3Upstream::open_qlog_file(const std::string_view &dir,
   if (fd != -1) {
     util::make_socket_closeonexec(fd);
   }
-#endif // !O_CLOEXEC
+#endif // !defined(O_CLOEXEC)
 
   if (fd == -1) {
     auto error = errno;
