@@ -30,7 +30,7 @@
 #include <cinttypes>
 #include <memory>
 
-#include "http-parser/http_parser.h"
+#include "llhttp.h"
 
 #include "shrpx_upstream.h"
 #include "memchunk.h"
@@ -78,7 +78,8 @@ public:
   virtual int on_downstream_reset(Downstream *downstream, bool no_retry);
   virtual int send_reply(Downstream *downstream, const uint8_t *body,
                          size_t bodylen);
-  virtual int initiate_push(Downstream *downstream, const StringRef &uri);
+  virtual int initiate_push(Downstream *downstream,
+                            const std::string_view &uri);
   virtual int response_riovec(struct iovec *iov, int iovcnt) const;
   virtual void response_drain(size_t n);
   virtual bool response_empty() const;
@@ -100,7 +101,7 @@ public:
 
 private:
   ClientHandler *handler_;
-  http_parser htp_;
+  llhttp_t htp_;
   size_t current_header_length_;
   std::unique_ptr<Downstream> downstream_;
   IOControl ioctrl_;
@@ -110,4 +111,4 @@ private:
 
 } // namespace shrpx
 
-#endif // SHRPX_HTTPS_UPSTREAM_H
+#endif // !defined(SHRPX_HTTPS_UPSTREAM_H)
