@@ -32,7 +32,8 @@
 #include "shrpx_memcached_request.h"
 #include "shrpx_memcached_result.h"
 #include "shrpx_config.h"
-#include "shrpx_ssl.h"
+#include "shrpx_tls.h"
+#include "shrpx_log.h"
 #include "util.h"
 
 namespace shrpx {
@@ -97,29 +98,31 @@ constexpr auto read_timeout = 10_s;
 
 MemcachedConnection::MemcachedConnection(const Address *addr,
                                          struct ev_loop *loop, SSL_CTX *ssl_ctx,
-                                         const StringRef &sni_name,
+                                         const std::string_view &sni_name,
                                          MemchunkPool *mcpool,
                                          std::mt19937 &gen)
-    : conn_(loop, -1, nullptr, mcpool, write_timeout, read_timeout, {}, {},
-            connectcb, readcb, timeoutcb, this, 0, 0., PROTO_MEMCACHED),
-      do_read_(&MemcachedConnection::noop),
-      do_write_(&MemcachedConnection::noop),
-      sni_name_(sni_name),
-      connect_blocker_(gen, loop, [] {}, [] {}),
-      parse_state_{},
-      addr_(addr),
-      ssl_ctx_(ssl_ctx),
-      sendsum_(0),
-      try_count_(0),
-      connected_(false) {}
+  : conn_(loop, -1, nullptr, mcpool, write_timeout, read_timeout, {}, {},
+          connectcb, readcb, timeoutcb, this, 0, 0., Proto::MEMCACHED),
+    do_read_(&MemcachedConnection::noop),
+    do_write_(&MemcachedConnection::noop),
+    sni_name_(sni_name),
+    connect_blocker_(
+      gen, loop, [] {}, [] {}),
+    parse_state_{},
+    addr_(addr),
+    ssl_ctx_(ssl_ctx),
+    sendsum_(0),
+    try_count_(0),
+    connected_(false) {}
 
-MemcachedConnection::~MemcachedConnection() { disconnect(); }
+MemcachedConnection::~MemcachedConnection() { conn_.disconnect(); }
 
 namespace {
 void clear_request(std::deque<std::unique_ptr<MemcachedRequest>> &q) {
   for (auto &req : q) {
     if (req->cb) {
-      req->cb(req.get(), MemcachedResult(MEMCACHED_ERR_EXT_NETWORK_ERROR));
+      req->cb(req.get(),
+              MemcachedResult(MemcachedStatusCode::EXT_NETWORK_ERROR));
     }
   }
   q.clear();
@@ -149,11 +152,12 @@ int MemcachedConnection::initiate_connection() {
   assert(conn_.fd == -1);
 
   if (ssl_ctx_) {
-    auto ssl = ssl::create_ssl(ssl_ctx_);
+    auto ssl = tls::create_ssl(ssl_ctx_);
     if (!ssl) {
       return -1;
     }
     conn_.set_ssl(ssl);
+    conn_.tls.client_session_cache = &tls_session_cache_;
   }
 
   conn_.fd = util::create_nonblock_socket(addr_->su.storage.ss_family);
@@ -178,11 +182,11 @@ int MemcachedConnection::initiate_connection() {
   }
 
   if (ssl_ctx_) {
-    if (!util::numeric_host(sni_name_.c_str())) {
-      SSL_set_tlsext_host_name(conn_.tls.ssl, sni_name_.c_str());
+    if (!util::numeric_host(sni_name_.data())) {
+      SSL_set_tlsext_host_name(conn_.tls.ssl, sni_name_.data());
     }
 
-    auto session = ssl::reuse_tls_session(tls_session_cache_);
+    auto session = tls::reuse_tls_session(tls_session_cache_);
     if (session) {
       SSL_set_session(conn_.tls.ssl, session);
       SSL_SESSION_free(session);
@@ -255,7 +259,7 @@ int MemcachedConnection::on_read() { return do_read_(*this); }
 int MemcachedConnection::tls_handshake() {
   ERR_clear_error();
 
-  conn_.last_read = ev_now(conn_.loop);
+  conn_.last_read = std::chrono::steady_clock::now();
 
   auto rv = conn_.tls_handshake();
   if (rv == SHRPX_ERR_INPROGRESS) {
@@ -274,19 +278,11 @@ int MemcachedConnection::tls_handshake() {
   auto &tlsconf = get_config()->tls;
 
   if (!tlsconf.insecure &&
-      ssl::check_cert(conn_.tls.ssl, addr_, sni_name_) != 0) {
+      tls::check_cert(conn_.tls.ssl, addr_, sni_name_) != 0) {
     connect_blocker_.on_failure();
     return -1;
   }
 
-  if (!SSL_session_reused(conn_.tls.ssl)) {
-    auto tls_session = SSL_get0_session(conn_.tls.ssl);
-    if (tls_session) {
-      ssl::try_cache_tls_session(tls_session_cache_, *addr_, tls_session,
-                                 ev_now(conn_.loop));
-    }
-  }
-
   ev_timer_stop(conn_.loop, &conn_.rt);
   ev_timer_stop(conn_.loop, &conn_.wt);
 
@@ -305,24 +301,28 @@ int MemcachedConnection::write_tls() {
     return 0;
   }
 
-  conn_.last_read = ev_now(conn_.loop);
+  conn_.last_read = std::chrono::steady_clock::now();
 
   std::array<struct iovec, MAX_WR_IOVCNT> iov;
   std::array<uint8_t, 16_k> buf;
 
   for (; !sendq_.empty();) {
     auto iovcnt = fill_request_buffer(iov.data(), iov.size());
-    auto p = std::begin(buf);
+    auto p = std::ranges::begin(buf);
     for (size_t i = 0; i < iovcnt; ++i) {
       auto &v = iov[i];
-      auto n = std::min(static_cast<size_t>(std::end(buf) - p), v.iov_len);
-      p = std::copy_n(static_cast<uint8_t *>(v.iov_base), n, p);
-      if (p == std::end(buf)) {
+      auto n =
+        std::min(static_cast<size_t>(std::ranges::end(buf) - p), v.iov_len);
+      p =
+        std::ranges::copy_n(static_cast<uint8_t *>(v.iov_base), as_signed(n), p)
+          .out;
+      if (p == std::ranges::end(buf)) {
         break;
       }
     }
 
-    auto nwrite = conn_.write_tls(buf.data(), p - std::begin(buf));
+    auto nwrite =
+      conn_.write_tls(buf.data(), as_unsigned(p - std::ranges::begin(buf)));
     if (nwrite < 0) {
       return -1;
     }
@@ -330,7 +330,7 @@ int MemcachedConnection::write_tls() {
       return 0;
     }
 
-    drain_send_queue(nwrite);
+    drain_send_queue(as_unsigned(nwrite));
   }
 
   conn_.wlimit.stopw();
@@ -344,7 +344,7 @@ int MemcachedConnection::read_tls() {
     return 0;
   }
 
-  conn_.last_read = ev_now(conn_.loop);
+  conn_.last_read = std::chrono::steady_clock::now();
 
   for (;;) {
     auto nread = conn_.read_tls(recvbuf_.last, recvbuf_.wleft());
@@ -357,7 +357,7 @@ int MemcachedConnection::read_tls() {
       return -1;
     }
 
-    recvbuf_.write(nread);
+    recvbuf_.write(as_unsigned(nread));
 
     if (parse_packet() != 0) {
       return -1;
@@ -372,13 +372,13 @@ int MemcachedConnection::write_clear() {
     return 0;
   }
 
-  conn_.last_read = ev_now(conn_.loop);
+  conn_.last_read = std::chrono::steady_clock::now();
 
   std::array<struct iovec, MAX_WR_IOVCNT> iov;
 
   for (; !sendq_.empty();) {
     auto iovcnt = fill_request_buffer(iov.data(), iov.size());
-    auto nwrite = conn_.writev_clear(iov.data(), iovcnt);
+    auto nwrite = conn_.writev_clear(iov.data(), static_cast<int>(iovcnt));
     if (nwrite < 0) {
       return -1;
     }
@@ -386,7 +386,7 @@ int MemcachedConnection::write_clear() {
       return 0;
     }
 
-    drain_send_queue(nwrite);
+    drain_send_queue(as_unsigned(nwrite));
   }
 
   conn_.wlimit.stopw();
@@ -400,7 +400,7 @@ int MemcachedConnection::read_clear() {
     return 0;
   }
 
-  conn_.last_read = ev_now(conn_.loop);
+  conn_.last_read = std::chrono::steady_clock::now();
 
   for (;;) {
     auto nread = conn_.read_clear(recvbuf_.last, recvbuf_.wleft());
@@ -413,7 +413,7 @@ int MemcachedConnection::read_clear() {
       return -1;
     }
 
-    recvbuf_.write(nread);
+    recvbuf_.write(as_unsigned(nread));
 
     if (parse_packet() != 0) {
       return -1;
@@ -430,15 +430,15 @@ int MemcachedConnection::parse_packet() {
     auto busy = false;
 
     switch (parse_state_.state) {
-    case MEMCACHED_PARSE_HEADER24: {
+    case MemcachedParseState::HEADER24: {
       if (recvbuf_.last - in < 24) {
-        recvbuf_.drain_reset(in - recvbuf_.pos);
+        recvbuf_.drain_reset(as_unsigned(in - recvbuf_.pos));
         return 0;
       }
 
       if (recvq_.empty()) {
         MCLOG(WARN, this)
-            << "Response received, but there is no in-flight request.";
+          << "Response received, but there is no in-flight request.";
         return -1;
       }
 
@@ -451,13 +451,14 @@ int MemcachedConnection::parse_packet() {
       }
       ++in;
 
-      parse_state_.op = *in++;
+      parse_state_.op = static_cast<MemcachedOp>(*in++);
       parse_state_.keylen = util::get_uint16(in);
       in += 2;
       parse_state_.extralen = *in++;
       // skip 1 byte reserved data type
       ++in;
-      parse_state_.status_code = util::get_uint16(in);
+      parse_state_.status_code =
+        static_cast<MemcachedStatusCode>(util::get_uint16(in));
       in += 2;
       parse_state_.totalbody = util::get_uint32(in);
       in += 4;
@@ -468,8 +469,9 @@ int MemcachedConnection::parse_packet() {
 
       if (req->op != parse_state_.op) {
         MCLOG(WARN, this)
-            << "opcode in response does not match to the request: want "
-            << static_cast<uint32_t>(req->op) << ", got " << parse_state_.op;
+          << "opcode in response does not match to the request: want "
+          << static_cast<uint32_t>(req->op) << ", got "
+          << static_cast<uint32_t>(parse_state_.op);
         return -1;
       }
 
@@ -485,8 +487,9 @@ int MemcachedConnection::parse_packet() {
         return -1;
       }
 
-      if (parse_state_.op == MEMCACHED_OP_GET &&
-          parse_state_.status_code == 0 && parse_state_.extralen == 0) {
+      if (parse_state_.op == MemcachedOp::GET &&
+          parse_state_.status_code == MemcachedStatusCode::NO_ERROR &&
+          parse_state_.extralen == 0) {
         MCLOG(WARN, this) << "response for GET does not have extra";
         return -1;
       }
@@ -500,17 +503,17 @@ int MemcachedConnection::parse_packet() {
       }
 
       if (parse_state_.extralen) {
-        parse_state_.state = MEMCACHED_PARSE_EXTRA;
+        parse_state_.state = MemcachedParseState::EXTRA;
         parse_state_.read_left = parse_state_.extralen;
       } else {
-        parse_state_.state = MEMCACHED_PARSE_VALUE;
-        parse_state_.read_left = parse_state_.totalbody - parse_state_.keylen -
-                                 parse_state_.extralen;
+        parse_state_.state = MemcachedParseState::VALUE;
+        parse_state_.read_left =
+          parse_state_.totalbody - parse_state_.keylen - parse_state_.extralen;
       }
       busy = true;
       break;
     }
-    case MEMCACHED_PARSE_EXTRA: {
+    case MemcachedParseState::EXTRA: {
       // We don't use extra for now. Just read and forget.
       auto n = std::min(static_cast<size_t>(recvbuf_.last - in),
                         parse_state_.read_left);
@@ -521,19 +524,20 @@ int MemcachedConnection::parse_packet() {
         recvbuf_.reset();
         return 0;
       }
-      parse_state_.state = MEMCACHED_PARSE_VALUE;
+      parse_state_.state = MemcachedParseState::VALUE;
       // since we require keylen == 0, totalbody - extralen ==
       // valuelen
       parse_state_.read_left =
-          parse_state_.totalbody - parse_state_.keylen - parse_state_.extralen;
+        parse_state_.totalbody - parse_state_.keylen - parse_state_.extralen;
       busy = true;
       break;
     }
-    case MEMCACHED_PARSE_VALUE: {
+    case MemcachedParseState::VALUE: {
       auto n = std::min(static_cast<size_t>(recvbuf_.last - in),
                         parse_state_.read_left);
 
-      parse_state_.value.insert(std::end(parse_state_.value), in, in + n);
+      parse_state_.value.insert(std::ranges::end(parse_state_.value), in,
+                                in + n);
 
       parse_state_.read_left -= n;
       in += n;
@@ -543,9 +547,9 @@ int MemcachedConnection::parse_packet() {
       }
 
       if (LOG_ENABLED(INFO)) {
-        if (parse_state_.status_code) {
+        if (parse_state_.status_code != MemcachedStatusCode::NO_ERROR) {
           MCLOG(INFO, this) << "response returned error status: "
-                            << parse_state_.status_code;
+                            << static_cast<uint16_t>(parse_state_.status_code);
         }
       }
 
@@ -584,9 +588,9 @@ int MemcachedConnection::parse_packet() {
 #define DEFAULT_WR_IOVCNT 128
 
 #if defined(IOV_MAX) && IOV_MAX < DEFAULT_WR_IOVCNT
-#define MAX_WR_IOVCNT IOV_MAX
+#  define MAX_WR_IOVCNT IOV_MAX
 #else // !defined(IOV_MAX) || IOV_MAX >= DEFAULT_WR_IOVCNT
-#define MAX_WR_IOVCNT DEFAULT_WR_IOVCNT
+#  define MAX_WR_IOVCNT DEFAULT_WR_IOVCNT
 #endif // !defined(IOV_MAX) || IOV_MAX >= DEFAULT_WR_IOVCNT
 
 size_t MemcachedConnection::fill_request_buffer(struct iovec *iov,
@@ -623,7 +627,7 @@ size_t MemcachedConnection::fill_request_buffer(struct iovec *iov,
     }
     if (buf.send_value_left) {
       iov[iovcnt++] = {req->value.data() + req->value.size() -
-                           buf.send_value_left,
+                         buf.send_value_left,
                        buf.send_value_left};
     }
   }
@@ -667,9 +671,9 @@ void MemcachedConnection::drain_send_queue(size_t nwrite) {
 
 size_t MemcachedConnection::serialized_size(MemcachedRequest *req) {
   switch (req->op) {
-  case MEMCACHED_OP_GET:
+  case MemcachedOp::GET:
     return 24 + req->key.size();
-  case MEMCACHED_OP_ADD:
+  case MemcachedOp::ADD:
   default:
     return 24 + 8 + req->key.size() + req->value.size();
   }
@@ -679,20 +683,21 @@ void MemcachedConnection::make_request(MemcachedSendbuf *sendbuf,
                                        MemcachedRequest *req) {
   auto &headbuf = sendbuf->headbuf;
 
-  std::fill(std::begin(headbuf.buf), std::end(headbuf.buf), 0);
+  std::ranges::fill(headbuf.buf, 0);
 
   headbuf[0] = MEMCACHED_REQ_MAGIC;
-  headbuf[1] = req->op;
+  headbuf[1] = static_cast<uint8_t>(req->op);
   switch (req->op) {
-  case MEMCACHED_OP_GET:
-    util::put_uint16be(&headbuf[2], req->key.size());
-    util::put_uint32be(&headbuf[8], req->key.size());
+  case MemcachedOp::GET:
+    util::put_uint16be(&headbuf[2], static_cast<uint16_t>(req->key.size()));
+    util::put_uint32be(&headbuf[8], static_cast<uint32_t>(req->key.size()));
     headbuf.write(24);
     break;
-  case MEMCACHED_OP_ADD:
-    util::put_uint16be(&headbuf[2], req->key.size());
+  case MemcachedOp::ADD:
+    util::put_uint16be(&headbuf[2], static_cast<uint16_t>(req->key.size()));
     headbuf[4] = 8;
-    util::put_uint32be(&headbuf[8], 8 + req->key.size() + req->value.size());
+    util::put_uint32be(&headbuf[8], static_cast<uint32_t>(8 + req->key.size() +
+                                                          req->value.size()));
     util::put_uint32be(&headbuf[28], req->expiry);
     headbuf.write(32);
     break;
@@ -755,18 +760,21 @@ void MemcachedConnection::reconnect_or_fail() {
                       << recvq_.size() + sendq_.size() << " request(s) again";
   }
 
-  q.insert(std::end(q), std::make_move_iterator(std::begin(recvq_)),
-           std::make_move_iterator(std::end(recvq_)));
-  q.insert(std::end(q), std::make_move_iterator(std::begin(sendq_)),
-           std::make_move_iterator(std::end(sendq_)));
+  q.insert(std::ranges::end(q),
+           std::make_move_iterator(std::ranges::begin(recvq_)),
+           std::make_move_iterator(std::ranges::end(recvq_)));
+  q.insert(std::ranges::end(q),
+           std::make_move_iterator(std::ranges::begin(sendq_)),
+           std::make_move_iterator(std::ranges::end(sendq_)));
 
   recvq_.clear();
   sendq_.clear();
 
   disconnect();
 
-  sendq_.insert(std::end(sendq_), std::make_move_iterator(std::begin(q)),
-                std::make_move_iterator(std::end(q)));
+  sendq_.insert(std::ranges::end(sendq_),
+                std::make_move_iterator(std::ranges::begin(q)),
+                std::make_move_iterator(std::ranges::end(q)));
 
   if (initiate_connection() != 0) {
     connect_blocker_.on_failure();
