@@ -36,10 +36,18 @@
 #include <queue>
 #ifndef NOTHREADS
 #  include <future>
-#endif // NOTHREADS
+#endif // !defined(NOTHREADS)
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
+#include "ssl_compat.h"
+
+#ifdef NGHTTP2_OPENSSL_IS_WOLFSSL
+#  include <wolfssl/options.h>
+#  include <wolfssl/openssl/ssl.h>
+#  include <wolfssl/openssl/err.h>
+#else // !defined(NGHTTP2_OPENSSL_IS_WOLFSSL)
+#  include <openssl/ssl.h>
+#  include <openssl/err.h>
+#endif // !defined(NGHTTP2_OPENSSL_IS_WOLFSSL)
 
 #include <ev.h>
 
@@ -53,7 +61,7 @@
 #ifdef ENABLE_HTTP3
 #  include "shrpx_quic_connection_handler.h"
 #  include "shrpx_quic.h"
-#endif // ENABLE_HTTP3
+#endif // defined(ENABLE_HTTP3)
 #include "allocator.h"
 
 using namespace nghttp2;
@@ -62,12 +70,12 @@ namespace shrpx {
 
 class Http2Session;
 class ConnectBlocker;
-class MemcachedDispatcher;
 struct UpstreamAddr;
 class ConnectionHandler;
+class AcceptHandler;
 #ifdef ENABLE_HTTP3
 class QUICListener;
-#endif // ENABLE_HTTP3
+#endif // defined(ENABLE_HTTP3)
 
 #ifdef HAVE_MRUBY
 namespace mruby {
@@ -75,7 +83,7 @@ namespace mruby {
 class MRubyContext;
 
 } // namespace mruby
-#endif // HAVE_MRUBY
+#endif // defined(HAVE_MRUBY)
 
 namespace tls {
 class CertLookupTree;
@@ -87,15 +95,15 @@ struct DownstreamAddr {
   Address addr;
   // backend address.  If |host_unix| is true, this is UNIX domain
   // socket path.
-  StringRef host;
-  StringRef hostport;
+  std::string_view host;
+  std::string_view hostport;
   // backend port.  0 if |host_unix| is true.
   uint16_t port;
   // true if |host| contains UNIX domain socket path.
   bool host_unix;
 
   // sni field to send remote server if TLS is enabled.
-  StringRef sni;
+  std::string_view sni;
 
   std::unique_ptr<ConnectBlocker> connect_blocker;
   std::unique_ptr<LiveCheck> live_check;
@@ -129,7 +137,7 @@ struct DownstreamAddr {
   // 256], inclusive.
   uint32_t weight;
   // name of group which this address belongs to.
-  StringRef group;
+  std::string_view group;
   // Weight of the weight group which this address belongs to.  Its
   // range is [1, 256], inclusive.
   uint32_t group_weight;
@@ -148,7 +156,7 @@ struct DownstreamAddr {
   bool queued;
 };
 
-constexpr uint32_t MAX_DOWNSTREAM_ADDR_WEIGHT = 256;
+inline constexpr uint32_t MAX_DOWNSTREAM_ADDR_WEIGHT = 256;
 
 struct DownstreamAddrEntry {
   DownstreamAddr *addr;
@@ -170,7 +178,8 @@ struct DownstreamAddrEntryGreater {
 struct WeightGroup {
   std::priority_queue<DownstreamAddrEntry, std::vector<DownstreamAddrEntry>,
                       DownstreamAddrEntryGreater>
-      pq;
+    pq;
+  std::string_view name;
   size_t seq;
   uint32_t weight;
   uint32_t cycle;
@@ -198,11 +207,11 @@ struct WeightGroupEntryGreater {
 
 struct SharedDownstreamAddr {
   SharedDownstreamAddr()
-      : balloc(1024, 1024),
-        affinity{SessionAffinity::NONE},
-        redirect_if_not_tls{false},
-        dnf{false},
-        timeout{} {}
+    : balloc(1024, 1024),
+      affinity{SessionAffinity::NONE},
+      redirect_if_not_tls{false},
+      dnf{false},
+      timeout{} {}
 
   SharedDownstreamAddr(const SharedDownstreamAddr &) = delete;
   SharedDownstreamAddr(SharedDownstreamAddr &&) = delete;
@@ -214,7 +223,7 @@ struct SharedDownstreamAddr {
   std::vector<WeightGroup> wgs;
   std::priority_queue<WeightGroupEntry, std::vector<WeightGroupEntry>,
                       WeightGroupEntryGreater>
-      pq;
+    pq;
   // Bunch of session affinity hash.  Only used if affinity ==
   // SessionAffinity::IP.
   std::vector<AffinityHash> affinity_hash;
@@ -223,7 +232,7 @@ struct SharedDownstreamAddr {
   std::unordered_map<uint32_t, size_t> affinity_hash_map;
 #ifdef HAVE_MRUBY
   std::shared_ptr<mruby::MRubyContext> mruby_ctx;
-#endif // HAVE_MRUBY
+#endif // defined(HAVE_MRUBY)
   // Configuration for session affinity
   AffinityConfig affinity;
   // Session affinity
@@ -265,12 +274,12 @@ struct WorkerStat {
 struct QUICPacket {
   QUICPacket(size_t upstream_addr_index, const Address &remote_addr,
              const Address &local_addr, const ngtcp2_pkt_info &pi,
-             const uint8_t *data, size_t datalen)
-      : upstream_addr_index{upstream_addr_index},
-        remote_addr{remote_addr},
-        local_addr{local_addr},
-        pi{pi},
-        data{data, data + datalen} {}
+             std::span<const uint8_t> data)
+    : upstream_addr_index{upstream_addr_index},
+      remote_addr{remote_addr},
+      local_addr{local_addr},
+      pi{pi},
+      data{std::ranges::begin(data), std::ranges::end(data)} {}
   QUICPacket() : upstream_addr_index{}, remote_addr{}, local_addr{}, pi{} {}
   size_t upstream_addr_index;
   Address remote_addr;
@@ -278,46 +287,34 @@ struct QUICPacket {
   ngtcp2_pkt_info pi;
   std::vector<uint8_t> data;
 };
-#endif // ENABLE_HTTP3
+#endif // defined(ENABLE_HTTP3)
 
 enum class WorkerEventType {
-  NEW_CONNECTION = 0x01,
   REOPEN_LOG = 0x02,
   GRACEFUL_SHUTDOWN = 0x03,
   REPLACE_DOWNSTREAM = 0x04,
 #ifdef ENABLE_HTTP3
   QUIC_PKT_FORWARD = 0x05,
-#endif // ENABLE_HTTP3
+#endif // defined(ENABLE_HTTP3)
 };
 
 struct WorkerEvent {
   WorkerEventType type;
-  struct {
-    sockaddr_union client_addr;
-    size_t client_addrlen;
-    int client_fd;
-    const UpstreamAddr *faddr;
-  };
-  std::shared_ptr<TicketKeys> ticket_keys;
   std::shared_ptr<DownstreamConfig> downstreamconf;
 #ifdef ENABLE_HTTP3
   std::unique_ptr<QUICPacket> quic_pkt;
-#endif // ENABLE_HTTP3
+#endif // defined(ENABLE_HTTP3)
 };
 
 class Worker {
 public:
   Worker(struct ev_loop *loop, SSL_CTX *sv_ssl_ctx, SSL_CTX *cl_ssl_ctx,
-         SSL_CTX *tls_session_cache_memcached_ssl_ctx,
          tls::CertLookupTree *cert_tree,
 #ifdef ENABLE_HTTP3
          SSL_CTX *quic_sv_ssl_ctx, tls::CertLookupTree *quic_cert_tree,
          WorkerID wid,
-#  ifdef HAVE_LIBBPF
-         size_t index,
-#  endif // HAVE_LIBBPF
-#endif   // ENABLE_HTTP3
-         const std::shared_ptr<TicketKeys> &ticket_keys,
+#endif // defined(ENABLE_HTTP3)
+         size_t index, const std::shared_ptr<TicketKeys> &ticket_keys,
          ConnectionHandler *conn_handler,
          std::shared_ptr<DownstreamConfig> downstreamconf);
   ~Worker();
@@ -329,7 +326,7 @@ public:
   tls::CertLookupTree *get_cert_lookup_tree() const;
 #ifdef ENABLE_HTTP3
   tls::CertLookupTree *get_quic_cert_lookup_tree() const;
-#endif // ENABLE_HTTP3
+#endif // defined(ENABLE_HTTP3)
 
   // These 2 functions make a lock m_ to get/set ticket keys
   // atomically.
@@ -342,7 +339,7 @@ public:
   SSL_CTX *get_cl_ssl_ctx() const;
 #ifdef ENABLE_HTTP3
   SSL_CTX *get_quic_sv_ssl_ctx() const;
-#endif // ENABLE_HTTP3
+#endif // defined(ENABLE_HTTP3)
 
   void set_graceful_shutdown(bool f);
   bool get_graceful_shutdown() const;
@@ -350,15 +347,13 @@ public:
   MemchunkPool *get_mcpool();
   void schedule_clear_mcpool();
 
-  MemcachedDispatcher *get_session_cache_memcached_dispatcher();
-
   std::mt19937 &get_randgen();
 
 #ifdef HAVE_MRUBY
   int create_mruby_context();
 
   mruby::MRubyContext *get_mruby_context() const;
-#endif // HAVE_MRUBY
+#endif // defined(HAVE_MRUBY)
 
   std::vector<std::shared_ptr<DownstreamAddrGroup>> &
   get_downstream_addr_groups();
@@ -372,6 +367,14 @@ public:
 
   ConnectionHandler *get_connection_handler() const;
 
+  int setup_server_socket();
+  void delete_listener();
+  void accept_pending_connection();
+  int create_tcp_server_socket(UpstreamAddr &addr);
+  void enable_listener();
+  void disable_listener();
+  void sleep_listener(ev_tstamp t);
+
 #ifdef ENABLE_HTTP3
   QUICConnectionHandler *get_quic_connection_handler();
 
@@ -385,45 +388,49 @@ public:
   bool should_update_bpf_map() const;
 
   uint32_t compute_sk_index() const;
-#  endif // HAVE_LIBBPF
+#  endif // defined(HAVE_LIBBPF)
 
   int create_quic_server_socket(UpstreamAddr &addr);
 
   // Returns a pointer to UpstreamAddr which matches |local_addr|.
   const UpstreamAddr *find_quic_upstream_addr(const Address &local_addr);
-#endif // ENABLE_HTTP3
+#endif // defined(ENABLE_HTTP3)
 
   DNSTracker *get_dns_tracker();
 
+  int handle_connection(int fd, sockaddr *addr, socklen_t addrlen,
+                        const UpstreamAddr *faddr);
+
 private:
 #ifndef NOTHREADS
   std::future<void> fut_;
-#endif // NOTHREADS
-#if defined(ENABLE_HTTP3) && defined(HAVE_LIBBPF)
+#endif // !defined(NOTHREADS)
   // Unique index of this worker.
   size_t index_;
-#endif // ENABLE_HTTP3 && HAVE_LIBBPF
   std::mutex m_;
   std::deque<WorkerEvent> q_;
   std::mt19937 randgen_;
   ev_async w_;
   ev_timer mcpool_clear_timer_;
   ev_timer proc_wev_timer_;
+  ev_timer disable_listener_timer_;
   MemchunkPool mcpool_;
   WorkerStat worker_stat_;
   DNSTracker dns_tracker_;
 
+  std::vector<UpstreamAddr> upstream_addrs_;
+  std::vector<std::unique_ptr<AcceptHandler>> listeners_;
+
 #ifdef ENABLE_HTTP3
   WorkerID worker_id_;
   std::vector<UpstreamAddr> quic_upstream_addrs_;
   std::vector<std::unique_ptr<QUICListener>> quic_listeners_;
-#endif // ENABLE_HTTP3
+#endif // defined(ENABLE_HTTP3)
 
   std::shared_ptr<DownstreamConfig> downstreamconf_;
-  std::unique_ptr<MemcachedDispatcher> session_cache_memcached_dispatcher_;
 #ifdef HAVE_MRUBY
   std::unique_ptr<mruby::MRubyContext> mruby_ctx_;
-#endif // HAVE_MRUBY
+#endif // defined(HAVE_MRUBY)
   struct ev_loop *loop_;
 
   // Following fields are shared across threads if
@@ -437,12 +444,14 @@ private:
   tls::CertLookupTree *quic_cert_tree_;
 
   QUICConnectionHandler quic_conn_handler_;
-#endif // ENABLE_HTTP3
+#endif // defined(ENABLE_HTTP3)
 
-#ifndef HAVE_ATOMIC_STD_SHARED_PTR
+#ifdef HAVE_ATOMIC_STD_SHARED_PTR
+  std::atomic<std::shared_ptr<TicketKeys>> ticket_keys_;
+#else  // !defined(HAVE_ATOMIC_STD_SHARED_PTR)
   std::mutex ticket_keys_m_;
-#endif // !HAVE_ATOMIC_STD_SHARED_PTR
   std::shared_ptr<TicketKeys> ticket_keys_;
+#endif // !defined(HAVE_ATOMIC_STD_SHARED_PTR)
   std::vector<std::shared_ptr<DownstreamAddrGroup>> downstream_addr_groups_;
   // Worker level blocker for downstream connection.  For example,
   // this is used when file descriptor is exhausted.
@@ -458,10 +467,10 @@ private:
 // group.  The catch-all group index is given in |catch_all|.  All
 // patterns are given in |groups|.
 size_t match_downstream_addr_group(
-    const RouterConfig &routerconfig, const StringRef &hostport,
-    const StringRef &path,
-    const std::vector<std::shared_ptr<DownstreamAddrGroup>> &groups,
-    size_t catch_all, BlockAllocator &balloc);
+  const RouterConfig &routerconfig, const std::string_view &hostport,
+  const std::string_view &path,
+  const std::vector<std::shared_ptr<DownstreamAddrGroup>> &groups,
+  size_t catch_all, BlockAllocator &balloc);
 
 // Calls this function if connecting to backend failed.  |raddr| is
 // the actual address used to connect to backend, and it could be
@@ -470,4 +479,4 @@ void downstream_failure(DownstreamAddr *addr, const Address *raddr);
 
 } // namespace shrpx
 
-#endif // SHRPX_WORKER_H
+#endif // !defined(SHRPX_WORKER_H)
