@@ -22,15 +22,19 @@
  * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-#ifndef OPENSSL_COMPAT_H
-#define OPENSSL_COMPAT_H
+#ifndef SSL_COMPAT_H
+#define SSL_COMPAT_H
 
 #include "nghttp2_config.h"
 
 #include <cstdint>
 
 #ifdef HAVE_WOLFSSL
+#  include <wolfssl/options.h>
+#  include <wolfssl/openssl/ssl.h>
+
 #  define NGHTTP2_OPENSSL_IS_WOLFSSL
+
 using nghttp2_ssl_op_type = long;
 using nghttp2_ssl_proto_version_type = int;
 using nghttp2_ssl_key_length_type = int;
@@ -38,8 +42,13 @@ using nghttp2_ssl_stack_index_type = int;
 using nghttp2_ssl_timeout_type = uint32_t;
 using nghttp2_ssl_rand_length_type = int;
 using nghttp2_ssl_verify_host_length_type = unsigned int;
-#else // !HAVE_WOLFSSL
-#  include <openssl/opensslv.h>
+
+inline constexpr auto NGHTTP2_CERT_TYPE_ECDSA = ECDSAk;
+inline constexpr auto NGHTTP2_CERT_TYPE_ML_DSA_44 = ML_DSA_LEVEL2k;
+inline constexpr auto NGHTTP2_CERT_TYPE_ML_DSA_65 = ML_DSA_LEVEL3k;
+inline constexpr auto NGHTTP2_CERT_TYPE_ML_DSA_87 = ML_DSA_LEVEL5k;
+#else // !defined(HAVE_WOLFSSL)
+#  include <openssl/ssl.h>
 
 #  ifdef LIBRESSL_VERSION_NUMBER
 #    define NGHTTP2_OPENSSL_IS_LIBRESSL
@@ -50,7 +59,7 @@ using nghttp2_ssl_stack_index_type = int;
 using nghttp2_ssl_timeout_type = long;
 using nghttp2_ssl_rand_length_type = int;
 using nghttp2_ssl_verify_host_length_type = size_t;
-#  endif // !LIBRESSL_VERSION_NUMBER
+#  endif // !defined(LIBRESSL_VERSION_NUMBER)
 
 #  if defined(OPENSSL_IS_BORINGSSL) || defined(OPENSSL_IS_AWSLC)
 #    define NGHTTP2_OPENSSL_IS_BORINGSSL
@@ -61,12 +70,13 @@ using nghttp2_ssl_stack_index_type = size_t;
 using nghttp2_ssl_timeout_type = uint32_t;
 using nghttp2_ssl_rand_length_type = size_t;
 using nghttp2_ssl_verify_host_length_type = size_t;
-#  endif // OPENSSL_IS_BORINGSSL || OPENSSL_IS_AWSLC
+#  endif // defined(OPENSSL_IS_BORINGSSL) || defined(OPENSSL_IS_AWSLC)
 
 #  if !defined(NGHTTP2_OPENSSL_IS_BORINGSSL) &&                                \
     !defined(NGHTTP2_OPENSSL_IS_LIBRESSL)
 #    define NGHTTP2_GENUINE_OPENSSL
-#  endif // !NGHTTP2_OPENSSL_IS_BORINGSSL && !NGHTTP2_OPENSSL_IS_LIBRESSL
+#  endif // !defined(NGHTTP2_OPENSSL_IS_BORINGSSL) &&
+         // !defined(NGHTTP2_OPENSSL_IS_LIBRESSL)
 
 #  ifdef NGHTTP2_GENUINE_OPENSSL
 #    define OPENSSL_3_0_0_API (OPENSSL_VERSION_NUMBER >= 0x30000000L)
@@ -82,10 +92,17 @@ using nghttp2_ssl_stack_index_type = int;
 using nghttp2_ssl_timeout_type = long;
 using nghttp2_ssl_rand_length_type = int;
 using nghttp2_ssl_verify_host_length_type = size_t;
-#  else    // !NGHTTP2_GENUINE_OPENSSL
+#  else    // !defined(NGHTTP2_GENUINE_OPENSSL)
 #    define OPENSSL_3_0_0_API 0
 #    define OPENSSL_3_5_0_API 0
-#  endif // !NGHTTP2_GENUINE_OPENSSL
-#endif   // !HAVE_WOLFSSL
+#  endif // !defined(NGHTTP2_GENUINE_OPENSSL)
 
-#endif // OPENSSL_COMPAT_H
+inline constexpr auto NGHTTP2_CERT_TYPE_ECDSA = EVP_PKEY_EC;
+#  if OPENSSL_3_5_0_API
+inline constexpr auto NGHTTP2_CERT_TYPE_ML_DSA_44 = EVP_PKEY_ML_DSA_44;
+inline constexpr auto NGHTTP2_CERT_TYPE_ML_DSA_65 = EVP_PKEY_ML_DSA_65;
+inline constexpr auto NGHTTP2_CERT_TYPE_ML_DSA_87 = EVP_PKEY_ML_DSA_87;
+#  endif // OPENSSL_3_5_0_API
+#endif   // !defined(HAVE_WOLFSSL)
+
+#endif // !defined(SSL_COMPAT_H)
