@@ -347,22 +347,18 @@ template <typename T, std::size_t N>
 [[nodiscard]] std::span<
   const uint8_t, N == std::dynamic_extent ? std::dynamic_extent : N * sizeof(T)>
 as_uint8_span(std::span<T, N> s) noexcept {
-  return std::span < const uint8_t,
-         N == std::dynamic_extent
-           ? std::dynamic_extent
-           : N * sizeof(T) >
-               {reinterpret_cast<const uint8_t *>(s.data()), s.size_bytes()};
+  return std::span<const uint8_t, N == std::dynamic_extent ? std::dynamic_extent
+                                                           : N * sizeof(T)>{
+    reinterpret_cast<const uint8_t *>(s.data()), s.size_bytes()};
 }
 
 template <typename T, std::size_t N>
 [[nodiscard]] std::span<uint8_t, N == std::dynamic_extent ? std::dynamic_extent
                                                           : N * sizeof(T)>
 as_writable_uint8_span(std::span<T, N> s) noexcept {
-  return std::span < uint8_t,
-         N == std::dynamic_extent
-           ? std::dynamic_extent
-           : N * sizeof(T) >
-               {reinterpret_cast<uint8_t *>(s.data()), s.size_bytes()};
+  return std::span<uint8_t, N == std::dynamic_extent ? std::dynamic_extent
+                                                     : N * sizeof(T)>{
+    reinterpret_cast<uint8_t *>(s.data()), s.size_bytes()};
 }
 
 template <typename R>
@@ -411,4 +407,4 @@ inline int run_app(std::function<int(int, char **)> app, int argc,
 
 } // namespace nghttp2
 
-#endif // TEMPLATE_H
+#endif // !defined(TEMPLATE_H)
