@@ -1,5 +1,5 @@
-/*
-** mruby/throw.h - mruby exception throwing handler
+/**
+** @file mruby/throw.h - mruby exception throwing handler
 **
 ** See Copyright Notice in mruby.h
 */
@@ -7,20 +7,24 @@
 #ifndef MRB_THROW_H
 #define MRB_THROW_H
 
-#if defined(MRB_ENABLE_CXX_ABI)
-# if !defined(__cplusplus)
+#if defined(MRB_USE_CXX_ABI) && !defined(__cplusplus)
 #  error Trying to use C++ exception handling in C code
-# endif
 #endif
 
-#if defined(MRB_ENABLE_CXX_EXCEPTION) && defined(__cplusplus)
+#if defined(MRB_USE_CXX_EXCEPTION)
+
+# if defined(__cplusplus)
 
 #define MRB_TRY(buf) try {
-#define MRB_CATCH(buf) } catch(mrb_jmpbuf_impl e) { if (e != (buf)->impl) { throw e; }
+#define MRB_CATCH(buf) } catch(mrb_jmpbuf *e) { if (e != (buf)) { throw e; }
 #define MRB_END_EXC(buf)  }
 
-#define MRB_THROW(buf) throw((buf)->impl)
-typedef mrb_int mrb_jmpbuf_impl;
+#define MRB_THROW(buf) throw(buf)
+typedef void *mrb_jmpbuf_impl;
+
+# else
+# error "need to be compiled with C++ compiler"
+# endif  /* __cplusplus */
 
 #else
 
@@ -29,6 +33,9 @@ typedef mrb_int mrb_jmpbuf_impl;
 #if defined(__APPLE__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
 #define MRB_SETJMP _setjmp
 #define MRB_LONGJMP _longjmp
+#elif defined(__MINGW64__) && defined(__GNUC__) && __GNUC__ >= 4
+#define MRB_SETJMP __builtin_setjmp
+#define MRB_LONGJMP __builtin_longjmp
 #else
 #define MRB_SETJMP setjmp
 #define MRB_LONGJMP longjmp
@@ -45,11 +52,6 @@ typedef mrb_int mrb_jmpbuf_impl;
 
 struct mrb_jmpbuf {
   mrb_jmpbuf_impl impl;
-
-#if defined(MRB_ENABLE_CXX_EXCEPTION) && defined(__cplusplus)
-  static mrb_int jmpbuf_id;
-  mrb_jmpbuf() : impl(jmpbuf_id++) {}
-#endif
 };
 
 #endif  /* MRB_THROW_H */
